// src/components/WeatherSection.tsx
import { FC } from "react";
import useSWR from "swr";
import CircularProgress from "@mui/material/CircularProgress";
import { WeatherData } from "../types/magic-mirror";
import WeatherIcon from "./WeatherIcons";

const WeatherSection: FC = () => {
  const { data, error } = useSWR<WeatherData>(
    ["weather", 116.3974, 39.9093],
    async ([, lon, lat]: [string, number, number]) => {
      const result = await window.electronAPI.getWeather({ lon, lat });
      if (result.error) throw new Error(result.message);
      return result;
    },
    {
      refreshInterval: 600000,
      onErrorRetry: (error) => {
        if (error.message.includes("超时")) return;
      },
    }
  );
  // 添加错误处理
  if (error) {
    return (
      <div className="absolute top-8 right-8 w-32 flex items-center justify-center">
        <span className="text-gray-400 text-sm">加载天气数据失败</span>
      </div>
    );
  }

  // 优化加载状态
  if (!data)
    return (
      <div className="absolute top-8 right-8 w-64 flex items-center justify-center">
        <CircularProgress size={24} sx={{ color: "#9CA3AF" }} />
        <span className="ml-2 text-gray-400">天气加载中...</span>
      </div>
    );

  return (
    <>
      {/* 当前天气模块 */}
      <div className="absolute top-8 right-0 w-64 space-y-4">
        <div className="grid grid-cols-2 gap-4 text-sm">
          <div className="space-y-1">
            <div className="text-gray-400">风速</div>
            <div className="text-gray-300">
              {data.realtime.wind.speed} {data.realtime.wind.direction}
            </div>
          </div>
          <div className="space-y-1">
            <div className="text-gray-400">空气质量</div>
            <div className="text-gray-300">
              {data.realtime.airQuality.aqi} (
              {data.realtime.airQuality.description})
            </div>
          </div>
        </div>

        <div className="flex items-center gap-4">
          <WeatherIcon type={data.realtime.skycon} className="!w-16 h-16" />
          <div>
            <div className="text-3xl">{data.realtime.temperature}°C</div>
            <div className="text-gray-400 text-sm">
              体感 {data.realtime.apparentTemperature}°C
            </div>
          </div>
        </div>
      </div>

      {/* 天气预报模块 */}
      {data.forecast && (
        <div className="absolute top-64 right-8 w-80 p-4">
          <div className="text-gray-200 text-sm font-medium mb-3">
            未来6天预报
          </div>
          <div className="space-y-3">
            {data.forecast.temperature.slice(0, 6).map((day, index) => (
              <div
                key={day.date}
                className="grid grid-cols-4 items-center gap-4 group transition-all duration-200 hover:bg-white/5 px-3 py-2 rounded-lg"
              >
                {/* 日期 */}
                <div className="text-gray-300 text-sm  w-14">
                  {new Date(day.date).toLocaleDateString("zh-CN", {
                    weekday: "short",
                  })}
                </div>

                {/* 天气图标和降水概率 */}
                <div className="flex items-center gap-2">
                  <WeatherIcon
                    type={data.forecast.skycon[index].value}
                    className="w-6 h-6"
                  />
                  <div className=" text-xs font-medium">
                    {data.forecast.precipitation[index].probability}%
                  </div>
                </div>

                {/* 温度条 */}
                <div className="flex-1">
                  <div className="relative h-1.5 bg-gray-700 rounded-full overflow-hidden">
                    <div
                      className="absolute inset-0 bg-gradient-to-r from-blue-400 to-red-400"
                      style={{
                        width: `${((day.max - day.min) / 30) * 100}%`,
                        left: `${((day.min - 0) / 30) * 100}%`,
                      }}
                    />
                  </div>
                </div>

                {/* 温度数值 */}
                <div className="flex items-center justify-between w-24">
                  <span className="text-blue-300 text-sm">
                    {Math.round(day.min)}°
                  </span>
                  <span className="text-gray-400 text-xs">→</span>
                  <span className="text-red-300 text-sm">
                    {Math.round(day.max)}°
                  </span>
                </div>
              </div>
            ))}
          </div>
        </div>
      )}
    </>
  );
};

export default WeatherSection;
